#include "testApp.h"



//--------------------------------------------------------------
void testApp::setup(){	 
	
	ofBackground(255,255,255);	
	
	// 0 output channels, 
	// 2 input channels
	// 22050 samples per second
	// 256 samples per buffer
	// 4 num buffers (latency)
	
	//CV
	#ifdef _USE_LIVE_VIDEO
		vidGrabber.setVerbose(true);
		vidGrabber.initGrabber(320,240);
		vidGrabber.getPixels();
	#else
		vidPlayer.loadMovie("fingers.mov");
		vidPlayer.play();
	#endif
		
    colorImg.allocate(320,240);
	grayImage.allocate(320,240);
	grayBg.allocate(320,240);
	grayDiff.allocate(320,240);
	grayDiffSmall.allocate(60,40);
	bLearnBakground = true;
	threshold = 30;
	bDrawDiagnostic = true;
	bForceOutward	= true; // do we push the particles away...

	
	ofSoundStreamSetup(0,2,this, 44100, 256, 4);	//void ofSoundStreamSetup(int nOutputChannels, int nInputChannels, ofBaseApp * OFSA, int sampleRate, int bufferSize, int nBuffers);
	left = new float[256];
	right = new float[256];
	
	
	for (int i = 0; i < 1000; i++){
		particle myParticle;
		myParticle.setInitialCondition(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()),0,0);
		myParticle.bounceOffWalls();
		particles.push_back(myParticle);
		
	}
	
	VF.setupField(60,40,ofGetWidth(), ofGetHeight());

	
	bufferCounter = 0;
	drawCounter = 0;

}
void testApp::update()
{

		//CV
		
		
	bool bNewFrame = false;
	
	#ifdef _USE_LIVE_VIDEO
		vidGrabber.grabFrame();
		bNewFrame = vidGrabber.isFrameNew();
	#else
		vidPlayer.idleMovie();
		bNewFrame = vidPlayer.isFrameNew();
	#endif
	
	if (bNewFrame){
		
		#ifdef _USE_LIVE_VIDEO
				colorImg.setFromPixels(vidGrabber.getPixels(), 320,240);
		#else
				colorImg.setFromPixels(vidPlayer.getPixels(), 320,240);
		#endif
		
        grayImage = colorImg;
		if (bLearnBakground == true){
			grayBg = grayImage;		// the = sign copys the pixels from grayImage into grayBg (operator overloading)
			bLearnBakground = false;
		}
		
		// take the abs value of the difference between background and incoming and then threshold:
		grayDiff.absDiff(grayBg, grayImage);
		grayDiff.threshold(threshold);
		grayDiffSmall.scaleIntoMe(grayDiff); //scales gray image into the vector field.
		grayDiffSmall.blur(5); // really blur the image alot!
		VF.setFromPixels(grayDiffSmall.getPixels(), bForceOutward, 0.005f); // bForceInward: determines whether going toward or away from white pixel.
	}
		bool bgotomiddle = false;
	    if(volume >= .07) bgotomiddle = true;

		for (int i = 0; i < particles.size(); i++){
		if(bgotomiddle)
		{
			particles[i].pos = particles[i].initpos;
		}
		particles[i].resetForce();
		// get the force from the vector field: 
		ofxVec2f frc;
		frc = VF.getForceFromPos(particles[i].pos.x, particles[i].pos.y);
		particles[i].addForce(frc.x, frc.y);
		particles[i].addDampingForce();
		particles[i].update();
		particles[i].bounceOffWalls();
		//particles[i].addRepulsionForce(frc.x, frc.y, float radius, float scale);
	
	}
}

//--------------------------------------------------------------
void testApp::draw(){
	grayImage.draw(0,0,ofGetWidth(),ofGetHeight());	


	for(int i = 0; i < particles.size(); i++)
	{

		particles[i].draw();
	}
	
	// draw the left:
//	ofSetColor(0x333333);
//	ofRect(100,100,256,200);
//	ofSetColor(0xFFFFFF);
//	for (int i = 0; i < 256; i++){
//		ofLine(100+i,200,100+i,200+left[i]*100.0f);
//	}
//	
//	// draw the right:
//	ofSetColor(0x333333);
//	ofRect(600,100,256,200);
//	ofSetColor(0xFFFFFF);
//	for (int i = 0; i < 256; i++){
//		ofLine(600+i,200,600+i,200+right[i]*100.0f);
//	}
//	
    // draw based on the averaged volume

	ofSetColor(255, 0, 0);
	ofCircle(490, 200, 100*volume);

//	ofSetColor(0x333333);
//	drawCounter++;
//	char reportString[255];
//	sprintf(reportString, "buffers received: %i\ndraw routines called: %i\n", bufferCounter,drawCounter);
//	ofDrawBitmapString(reportString,80,380);

	ofSetColor(255, 255, 0);
	VF.draw();
		
	ofSetColor(0xffffff);
}


static float adder = 0;
//--------------------------------------------------------------
//--------------------------------------------------------------
void testApp::audioReceived 	(float * input, int bufferSize, int nChannels){	
	// samples are "interleaved"
	volume = 0;   // averaged volume
	for (int i = 0; i < bufferSize; i++){
		
		volume += input[i*2]*input[i*2];  //volume is going from -1 to 1.
		left[i] = input[i*2];
		right[i] = input[i*2+1];
	}
	volume = sqrt(volume / (float)bufferSize); //volume mapping formular
	cout << volume << " >> volume"<<endl;
	bufferCounter++;
	
}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){ 
	
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}



